--- 模块功能：生成需要的数据
-- @author openLuat
-- @module LUAT_YUN_PAY_4G.dataPack
-- @license MIT
-- @copyright openLuat
-- @release 2018.10.11

module(...,package.seeall)
require"misc"
require"lbsLoc"
require"net"
require"socket"

local function bcd(d,n)
    local l = (d or ""):len()
    local num
    local s = ""
    for i=1,l,2 do
        num = tonumber(d:sub(i,i+1),16)
        num = i == l and 0xf0+num or (num%0x10)*0x10 + num/0x10
        s = s..string.char(num)
    end
    s = s:len() < n and s .. ("\255"):rep(n-l) or s:len() > n and s:sub(1,n) or s
    return s
end

local function enlnla(s)
	local v1,v2 = s:match("(%d+)%.(%d+)")
	if v1:len() < 3 then v1 = string.rep("0",3-v1:len())..v1 end
	return bcd(v1..v2,5)
end

--登陆报文
function login()
    local payload = pack.pack(">bbHHbHHbHAbHbbHAbHAbHA",
    14,
    0,2,24,
    1,2,300,
    2,2,bcd(_G.VERSION:gsub("%.",""),2),
    3,1,0--[[gpsupport and 1 or 0]],
    4,sim.getIccid():len(),sim.getIccid(),
    8,_G.PROJECT:len(),_G.PROJECT,
    13,sim.getImsi():len(),sim.getImsi())
    return payload, "/v1/device/"..misc.getImei().."/devdata"
end

local lastLoc = pack.pack(">bAA",7,string.fromHex("FFFFFFFFFF"),string.fromHex("FFFFFFFFFF"))
--位置报文
function location(lbsEnable)
    local payload
    if lbsEnable then
        lbsLoc.request(function (result,lat,lng)
            sys.publish("LBS_LOCATION",result==0,lat,lng)
        end)--请求基站定位
        local result,r,lat,lng = sys.waitUntil("LBS_LOCATION",60000)
        if not r then return location(false) end--定位失败
        payload = pack.pack(">bAA",7,enlnla(lng),enlnla(lat))
        lastLoc = payload
    else
        payload = lastLoc
    end
    payload = payload..pack.pack(">HbbbbbHbHb",0,0,0,0,0,
    net.getRssi() > 31 and 31 or net.getRssi(),
    misc.getVbatt() or 0,
    0x0e,1,
    socket.isReady() and (_G.moduleType == 4 and net.getNetMode() or 2) or 0)
    return payload, "/v1/device/"..misc.getImei().."/devdata"
end

--支付报文应答
--token     token字符串
--result    成功/失败
function payReply(token,result)
    log.info("dataPack.reply","pay reply",token,result)
    local payload = pack.pack(">bbb",0x0c,0x54,result and 1 or 0)
    return payload, "/v1/device/"..misc.getImei().."/deveventrsp/"..token
end

--投币统计上报
function coin(sum)
    local t = misc.getClock()
    local payload = pack.pack("bbbbbbbH",0x1b,
    t.year-2000,t.month,t.day,t.hour,t.min,t.sec,
    sum)
    return payload, "/v1/device/"..misc.getImei().."/coincheck"
end

--透传上报
function trans(token,result,backData)
    log.info("dataPack.reply","trans reply",token,result,backData)
    local payload = pack.pack(">bA",0x1c, backData and "#RecvCmd,"..backData:toHex().."#"
            or (result and "#RecvCmd,1#" or "#RecvCmd,0#"))
    return payload, "/v1/device/"..misc.getImei().."/c_trans/"..token
end

--上报继电器停止报文
function relayStop(normalStop)
    log.info("dataPack.relayStop","normalStop",normalStop)
    local payload = pack.pack("bbHb",0x21,0x00,1,(normalStop and 0x00 or 0x01))
    return payload, "/v1/device/"..misc.getImei().."/devmafurpt"
end



--登陆应答，设置时间
function set(payload)
    if payload:byte() ~= 0x3c or payload:byte(2) ~= 0x00 then return end
    misc.setClock({
        year  = payload:byte(3)+2000,
        month = payload:byte(4),
        day   = payload:byte(5),
        hour  = payload:byte(6),
        min   = payload:byte(7),
        sec   = payload:byte(8)})
    log.info("dataPack.set","time set",
    payload:byte(3)+2000,
    payload:byte(4),
    payload:byte(5),
    payload:byte(6),
    payload:byte(7),
    payload:byte(8))
    sys.publish("TIME_SET")--发布消息，时间已设置
end

--解析支付命令报文内容
--return payType,worktime,high,low,pluse
function unpackPay(payload)
    if payload:byte() ~= 0x54 then return end
    local _,payType,money,worktime,high,low,pluse = pack.unpack(payload:sub(2),">bIIIII")
    log.info("dataPack.unpack","pay unpack",payType,worktime,high,low,pluse,money)
    if payType == 5 then
        local year,month,day,hour,min,sec,_ = payload:byte(5),payload:byte(6),payload:byte(7),
                                              payload:byte(8),payload:byte(9),payload:byte(10)
        log.info("dataPack.unpack","day unpack",year+2000,month,day,hour,min,sec)
        local timeStamp = os.time({day=day, month=month, year=year+2000,
                             hour=hour, min=min, sec=sec})
        worktime = timeStamp - os.time()
    end
    return payType,worktime,high,low,pluse,money
end

--投币事件成功判断
function cionR(payload)
    if payload:byte() ~= 0x55 then return end
    return payload:byte(2) == 0
end

